﻿package 
{
	import flash.display.Sprite;
	import flash.display.Graphics;
	import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.events.MouseEvent;
	import flash.geom.Matrix;
	import flash.utils.Timer;
	import flash.ui.Mouse;
	import com.greensock.TweenMax;
	import com.greensock.easing.*;
	public class Application extends Sprite
	{

		private var ballPlayer:Ball;
		private var eballs:Array;
		private var tmr:Timer;
		private var score:Score;
		public function Application():void
		{
			init();
		}

		private function init():void
		{
			ballPlayer = new PlayerBall();
			eballs = new Array();
			tmr = new Timer(10);
			score = new Score();

			stage.align = "TL";
			stage.scaleMode = "noScale";
			Mouse.hide();

			setBackground();

			score.x = stage.stageWidth / 2;
			score.y = stage.stageHeight / 2;
			stage.addChild(score);

			stage.addEventListener(MouseEvent.MOUSE_MOVE, updatePlayerBall);
			stage.addChild(ballPlayer);

			tmr.addEventListener(TimerEvent.TIMER, updateTime);

			stage.addEventListener(MouseEvent.CLICK, startGame);
		}

		private function setBackground():void
		{
			var type:String = "radial";
			var colors:Array = [0xffffff,0xcccccc];
			var alphas:Array = [1,1];
			var ratios:Array = [0,255];
			var matr:Matrix = new Matrix();
			matr.createGradientBox(stage.stageWidth, stage.stageHeight, Math.PI / 2, 0, 0 );
			//SpreadMethod will define how the gradient is spread. Note!!! Flash uses CONSTANTS to represent String literals
			var sprMethod:String = "pad";
			//Start the Gradietn and pass our variables to it
			var sprite:Sprite = new Sprite();
			//Save typing + increase performance through local reference to a Graphics object
			var g:Graphics = sprite.graphics;
			g.beginGradientFill( type, colors, alphas, ratios, matr, sprMethod );
			g.drawRect(0,0,stage.stageWidth,stage.stageHeight);
			stage.addChild(sprite);
		}

		private function updatePlayerBall(e:MouseEvent):void
		{
			ballPlayer.x = mouseX;
			ballPlayer.y = mouseY;
		}

		private function updateTime(e:TimerEvent):void
		{
			score.txtScore.text = String(((tmr.currentCount*tmr.delay)/1000).toFixed(2));
			if ((tmr.currentCount*tmr.delay) % 5000 == 0)
			{
				addBall();
			}
		}

		private function startGame(e:MouseEvent):void
		{
			stage.removeEventListener(MouseEvent.CLICK, startGame);
			addBall();
			addBall();
			addBall();
			tmr.start();
			stage.addEventListener(Event.ENTER_FRAME, gameLoop);
		}

		private function addBall():void
		{
			var ball:Ball = new Ball(10,Math.random() * Math.PI * 2,5);
			ball.x = Math.random() * stage.stageWidth;
			ball.y = Math.random() * stage.stageHeight;
			ball.alpha = 0;
			stage.addChild(ball);
			TweenMax.to(ball, 0.5, {alpha:1});
			TweenMax.to(ball, 0, {delay: 1, onComplete:function():void{eballs.push(ball)}});
		}

		private function gameLoop(e:Event):void
		{
			for (var i:uint = 0; i < eballs.length; i++)
			{
				for (var j:uint = i + 1; j < eballs.length; j++)
				{
					if (collision(eballs[i],eballs[j]))
					{
						doCollision(eballs[i], eballs[j]);
					}
				}
	
				if (collision(eballs[i],ballPlayer))
				{
					endOfGame();
					break;
				}
	
				eballs[i].update();
				checkBounds(eballs[i]);
			}
		}
	
		private function collision(ball1:Ball, ball2:Ball):Boolean
		{
			var xDist:Number = ball1.x - ball2.x;
			var yDist:Number = ball1.y - ball2.y;
			var Dist:Number = Math.sqrt(xDist * xDist + yDist * yDist);
	
			return Dist <= ball1.radius + ball2.radius;
		}
	
		private function doCollision(ball1:Ball, ball2:Ball):void
		{
			var xDist:Number = ball1.x - ball2.x;
			var yDist:Number = ball1.y - ball2.y;
			var collisionAngle:Number = Math.atan2(yDist,xDist);
			var magBall1:Number = Math.sqrt(ball1.xSpeed * ball1.xSpeed + ball1.ySpeed * ball1.ySpeed);
			var magBall2:Number = Math.sqrt(ball2.xSpeed * ball2.xSpeed + ball2.ySpeed * ball2.ySpeed);
			var angleBall1:Number = Math.atan2(ball1.ySpeed,ball1.xSpeed);
			var angleBall2:Number = Math.atan2(ball2.ySpeed,ball2.xSpeed);
			var xSpeedBall1:Number = magBall1 * Math.cos(angleBall1 - collisionAngle);
			var ySpeedBall1:Number = magBall1 * Math.sin(angleBall1 - collisionAngle);
			var xSpeedBall2:Number = magBall2 * Math.cos(angleBall2 - collisionAngle);
			var ySpeedBall2:Number = magBall2 * Math.sin(angleBall2 - collisionAngle);
			var finalxSpeedBall1:Number = ((ball1.mass-ball2.mass)*xSpeedBall1+(ball2.mass+ball2.mass)*xSpeedBall2)/(ball1.mass+ball2.mass);
			var finalxSpeedBall2:Number = ((ball1.mass+ball1.mass)*xSpeedBall1+(ball2.mass-ball1.mass)*xSpeedBall2)/(ball1.mass+ball2.mass);
			var finalySpeedBall1:Number = ySpeedBall1;
			var finalySpeedBall2:Number = ySpeedBall2;
			ball1.xSpeed = Math.cos(collisionAngle) * finalxSpeedBall1 + Math.cos(collisionAngle + Math.PI / 2) * finalySpeedBall1;
			ball1.ySpeed = Math.sin(collisionAngle) * finalxSpeedBall1 + Math.sin(collisionAngle + Math.PI / 2) * finalySpeedBall1;
			ball2.xSpeed = Math.cos(collisionAngle) * finalxSpeedBall2 + Math.cos(collisionAngle + Math.PI / 2) * finalySpeedBall2;
			ball2.ySpeed = Math.sin(collisionAngle) * finalxSpeedBall2 + Math.sin(collisionAngle + Math.PI / 2) * finalySpeedBall2;
		}
	
		private function endOfGame():void
		{
			tmr.stop();
			Mouse.show();
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, updatePlayerBall);
			stage.removeEventListener(Event.ENTER_FRAME, gameLoop);
	
			while (eballs.length > 0)
			{
				TweenMax.to(eballs[0], 0.5, {scaleX:0, scaleY:0, ease:Bounce.easeOut});
				eballs.splice(0,1);
			}
	
			TweenMax.to(ballPlayer, 0.5, {scaleX:0, scaleY:0, ease:Bounce.easeOut});
		}
	
		private function checkBounds(ball:Ball):void
		{
			if ((ball.x + ball.radius) > stage.stageWidth)
			{
				ball.x = stage.stageWidth - ball.radius;
				ball.xSpeed *=  -1;
			}
			if ((ball.x - ball.radius) < 0)
			{
				ball.x = 0 + ball.radius;
				ball.xSpeed *=  -1;
			}
			if ((ball.y + ball.radius) > stage.stageHeight)
			{
				ball.y = stage.stageHeight - ball.radius;
				ball.ySpeed *=  -1;
			}
			if ((ball.y - ball.radius) < 0)
			{
				ball.y = 0 + ball.radius;
				ball.ySpeed *=  -1;
			}
		}
	}
}